@@ -16,8 +16,7 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import apt
-import aptdaemon
+import atexit
 import locale
 import dbus
 import dbus.service
@@ -33,7 +32,9 @@ import xapian
 
 from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 
+from softwarecenter import Application
 from softwarecenter.enums import *
+from softwarecenter.utils import *
 from softwarecenter.version import *
 from softwarecenter.db.database import StoreDatabase
 
@@ -41,9 +42,15 @@ import view.dialogs
 from view.viewswitcher import ViewSwitcher, ViewSwitcherList
 from view.pendingview import PendingView
 from view.installedpane import InstalledPane
+from view.channelpane import ChannelPane
 from view.availablepane import AvailablePane
 from view.softwarepane import SoftwarePane
 
+from backend.config import get_config
+from backend import get_install_backend
+
+from distro import get_distro
+
 from apt.aptcache import AptCache
 from gettext import gettext as _
 
@@ -67,7 +74,9 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
     
     (NOTEBOOK_PAGE_AVAILABLE,
      NOTEBOOK_PAGE_INSTALLED,
-     NOTEBOOK_PAGE_PENDING) = range(3)
+     NOTEBOOK_PAGE_SEPARATOR_1,
+     NOTEBOOK_PAGE_PENDING,
+     NOTEBOOK_PAGE_CHANNEL) = range(5)
 
     WEBLINK_URL = "http://apt.ubuntu.com/p/%s"
 
@@ -77,6 +86,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
                                      "software-center")
         gettext.bindtextdomain("software-center", "/usr/share/locale")
         gettext.textdomain("software-center")
+
         try:
             locale.setlocale(locale.LC_ALL, "")
         except:
@@ -92,10 +102,21 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         except Exception, e:
             logging.exception("setlocale failed")
 
+        # distro specific stuff
+        self.distro = get_distro()
+
+        # a main iteration friendly apt cache
+        self.cache = AptCache()
+        self.backend = get_install_backend()
+        self.backend.connect("transaction-started", self._on_transaction_started)
+        self.backend.connect("transaction-finished", self._on_transaction_finished)
+        self.backend.connect("transaction-stopped", self._on_transaction_stopped)
+        self.backend.connect("channels-changed", self.on_channels_changed)
+
         # xapian
         pathname = os.path.join(xapian_base_path, "xapian")
         try:
-            self.db = StoreDatabase(pathname)
+            self.db = StoreDatabase(pathname, self.cache)
             self.db.open()
         except xapian.DatabaseOpeningError:
             # Couldn't use that folder as a database
@@ -106,7 +127,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
                 from softwarecenter.db.update import rebuild_database
                 logging.info("building local database")
                 rebuild_database(pathname)
-                self.db = StoreDatabase(pathname)
+                self.db = StoreDatabase(pathname, self.cache)
                 self.db.open()
         except xapian.DatabaseCorruptError, e:
             logging.exception("xapian open failed")
@@ -120,44 +141,53 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         # additional icons come from app-install-data
         self.icons = gtk.icon_theme_get_default()
         self.icons.append_search_path(ICON_PATH)
-        self.icons.append_search_path(SOFTWARE_CENTER_ICON_PATH)
+        self.icons.append_search_path(os.path.join(datadir,"icons"))
+        self.icons.append_search_path(os.path.join(datadir,"emblems"))
         # HACK: make it more friendly for local installs (for mpt)
         self.icons.append_search_path(datadir+"/icons/32x32/status")
-        
-        # a main iteration friendly apt cache
-        self.cache = AptCache()
+        gtk.window_set_default_icon_name("softwarecenter")
 
         # misc state
-        self._pending_transactions = 0
         self._block_menuitem_view = False
         self._available_items_for_page = {}
-        # FIXME: make this all part of a application object
-        self._selected_pkgname_for_page = {}
-        self._selected_appname_for_page = {}
 
         # available pane
         self.available_pane = AvailablePane(self.cache, self.db,
+                                            self.distro,
                                             self.icons, datadir)
         self.available_pane.app_details.connect("selected", 
                                                 self.on_app_details_changed,
                                                 self.NOTEBOOK_PAGE_AVAILABLE)
         self.available_pane.app_view.connect("application-selected",
-                                             self.on_app_selected,
-                                             self.NOTEBOOK_PAGE_AVAILABLE)
+                                             self.on_app_selected)
         self.available_pane.connect("app-list-changed", 
                                     self.on_app_list_changed,
                                     self.NOTEBOOK_PAGE_AVAILABLE)
         self.alignment_available.add(self.available_pane)
 
+        # channel pane
+        self.channel_pane = ChannelPane(self.cache, self.db,
+                                            self.distro,
+                                            self.icons, datadir)
+        self.channel_pane.app_details.connect("selected", 
+                                                self.on_app_details_changed,
+                                                self.NOTEBOOK_PAGE_CHANNEL)
+        self.channel_pane.app_view.connect("application-selected",
+                                             self.on_app_selected)
+        self.channel_pane.connect("app-list-changed", 
+                                    self.on_app_list_changed,
+                                    self.NOTEBOOK_PAGE_CHANNEL)
+        self.alignment_channel.add(self.channel_pane)
+        
         # installed pane
         self.installed_pane = InstalledPane(self.cache, self.db,
+                                            self.distro,
                                             self.icons, datadir)
         self.installed_pane.app_details.connect("selected", 
                                                 self.on_app_details_changed,
                                                 self.NOTEBOOK_PAGE_INSTALLED)
         self.installed_pane.app_view.connect("application-selected",
-                                             self.on_app_selected,
-                                             self.NOTEBOOK_PAGE_INSTALLED)
+                                             self.on_app_selected)
         self.installed_pane.connect("app-list-changed", 
                                     self.on_app_list_changed,
                                     self.NOTEBOOK_PAGE_INSTALLED)
@@ -168,13 +198,11 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         self.scrolledwindow_transactions.add(self.pending_view)
 
         # view switcher
-        self.view_switcher = ViewSwitcher(datadir, self.icons)
+        self.view_switcher = ViewSwitcher(datadir, self.db, self.icons)
         self.scrolledwindow_viewswitcher.add(self.view_switcher)
         self.view_switcher.show()
         self.view_switcher.connect("view-changed", 
                                    self.on_view_switcher_changed)
-        self.view_switcher.model.connect("transactions-changed", 
-                                   self.on_view_switcher_transactions_changed)
         self.view_switcher.set_view(ViewSwitcherList.ACTION_ITEM_AVAILABLE)
 
         # launchpad integration help, its ok if that fails
@@ -187,50 +215,52 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
 
         # default focus
         self.available_pane.searchentry.grab_focus()
+        self.window_main.set_size_request(600, 400)
+
+        # restore state
+        self.config = get_config()
+        self.restore_state()
 
     # callbacks
-    def on_app_details_changed(self, widget, appname, pkgname, page):
-        self._selected_pkgname_for_page[page] = pkgname
-        self._selected_appname_for_page[page] = appname
+    def on_app_details_changed(self, widget, app, page):
         self.update_app_status_menu()
         self.update_status_bar()
 
     def on_app_list_changed(self, pane, new_len, page):
         self._available_items_for_page[page] = new_len
         if self.notebook_view.get_current_page() == page:
+            self.update_app_list_view()
+            self.update_app_status_menu()
             self.update_status_bar()
 
-    def on_app_selected(self, widget, appname, pkgname, page):
-        self._selected_appname_for_page[page] = appname
-        self._selected_pkgname_for_page[page] = pkgname
+    def on_app_selected(self, widget, app):
+        self.update_app_status_menu()
         self.menuitem_copy.set_sensitive(True)
-        self.menuitem_copy_web_link.set_sensitive(True)
 
     def on_window_main_delete_event(self, widget, event):
+        self.save_state()
         gtk.main_quit()
         
-    def on_view_switcher_transactions_changed(self, view_switcher, pending_nr):
-        # if the pending number drops to zero check if we should switch
-        # to the previous view
-        if pending_nr == 0 and self._view_before_pending_switch is not None:
-            if self.view_switcher.get_view() is None:
-                self.view_switcher.set_view(self._view_before_pending_switch)
-            self._view_before_pending_switch = None
-        # the spec says that on the first transaction it should auto-switch
-        # to the progress view
-        elif pending_nr > 0 and self._pending_transactions == 0:
-            self._view_before_pending_switch = self.view_switcher.get_view()
-            self.view_switcher.set_view(ViewSwitcherList.ACTION_ITEM_PENDING)
-        self._pending_transactions = pending_nr
-
-    def on_view_switcher_changed(self, view_switcher, action):
+    def on_window_main_key_press_event(self, widget, event):
+        if (event.keyval == gtk.gdk.keyval_from_name("BackSpace") and 
+            self.active_pane and
+            not self.active_pane.searchentry.is_focus()):
+            self.active_pane.navigation_bar.navigate_up()
+        
+    def on_view_switcher_changed(self, view_switcher, action, channel):
         logging.debug("view_switcher_activated: %s %s" % (view_switcher,action))
         if action == self.NOTEBOOK_PAGE_AVAILABLE:
             self.active_pane = self.available_pane
+        elif action == self.NOTEBOOK_PAGE_CHANNEL:
+            self.channel_pane.set_channel(channel)
+            self.active_pane = self.channel_pane
         elif action == self.NOTEBOOK_PAGE_INSTALLED:
             self.active_pane = self.installed_pane
         elif action == self.NOTEBOOK_PAGE_PENDING:
             self.active_pane = None
+        elif action == self.NOTEBOOK_PAGE_SEPARATOR_1:
+            # do nothing
+            return
         else:
             assert False, "Not reached"
         # set menu sensitve
@@ -239,22 +269,29 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         # set menu state
         if self.active_pane:
             self._block_menuitem_view = True
-            if self.active_pane.apps_filter.get_supported_only():
+            if not self.active_pane.apps_filter:
+                self.menuitem_view_all.set_sensitive(False)
+                self.menuitem_view_supported_only.set_sensitive(False)
+            elif self.active_pane.apps_filter.get_supported_only():
                 self.menuitem_view_supported_only.activate()
             else:
                 self.menuitem_view_all.activate()
             self._block_menuitem_view = False
         # switch to new page
         self.notebook_view.set_current_page(action)
+        self.update_app_list_view(channel)
         self.update_status_bar()
         self.update_app_status_menu()
 
     # Menu Items
-
     def on_menuitem_install_activate(self, menuitem):
+        app = self.active_pane.get_current_app()
+        self.active_pane.app_details.init_app(app)
         self.active_pane.app_details.install()
 
     def on_menuitem_remove_activate(self, menuitem):
+        app = self.active_pane.get_current_app()
+        self.active_pane.app_details.init_app(app)
         self.active_pane.app_details.remove()
         
     def on_menuitem_close_activate(self, widget):
@@ -266,6 +303,10 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         """
         if self.active_pane:
             state = self.active_pane.searchentry.is_focus()
+            if self.active_pane.searchentry.flags() & gtk.VISIBLE:
+                self.menuitem_search.set_sensitive(not state)
+            else:
+                self.menuitem_search.set_sensitive(False)
         else:
             state = False
         edit_menu_items = [self.menuitem_undo, 
@@ -300,13 +341,10 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         self.active_pane.searchentry.select_region(0, -1)
 
     def on_menuitem_copy_web_link_activate(self, menuitem):
-        page = self.notebook_view.get_current_page()
-        try:
-            pkg = self._selected_pkgname_for_page[page]
-        except KeyError, e:
-            return
-        clipboard = gtk.Clipboard()
-        clipboard.set_text(self.WEBLINK_URL % pkg)
+        app = self.active_pane.get_current_app()
+        if app:
+            clipboard = gtk.Clipboard()
+            clipboard.set_text(self.WEBLINK_URL % app.pkgname)
 
     def on_menuitem_search_activate(self, widget):
         if self.active_pane:
@@ -344,47 +382,50 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         # run yelp
         p = subprocess.Popen(["yelp","ghelp:software-center"])
         # collect the exit status (otherwise we leave zombies)
-        glib.timeout_add(1000, lambda p: p.poll() == None, p)
+        glib.timeout_add_seconds(1, lambda p: p.poll() == None, p)
 
     def on_menuitem_view_all_activate(self, widget):
-        if self._block_menuitem_view:
-            return
-        self.active_pane.apps_filter.set_supported_only(False)
-        self.active_pane.refresh_apps()
+        if not self._block_menuitem_view and self.active_pane.apps_filter.get_supported_only():
+            self.active_pane.apps_filter.set_supported_only(False)
+            self.active_pane.refresh_apps()
 
     def on_menuitem_view_supported_only_activate(self, widget):
-        if self._block_menuitem_view: 
-            return
-        self.active_pane.apps_filter.set_supported_only(True)
-        self.active_pane.refresh_apps()
+        if not self._block_menuitem_view and not self.active_pane.apps_filter.get_supported_only():
+            self.active_pane.apps_filter.set_supported_only(True)
+            self.active_pane.refresh_apps()
+            
+    def _on_transaction_started(self, backend):
+        self.menuitem_install.set_sensitive(False)
+        self.menuitem_remove.set_sensitive(False)
+            
+    def _on_transaction_finished(self, backend, success):
+        """ callback when an application install/remove transaction has finished """
+        self.cache.open()
+        self.update_app_status_menu()
+
+    def _on_transaction_stopped(self, backend):
+        """ callback when an application install/remove transaction has stopped """
+        self.update_app_status_menu()
+
+    def on_channels_changed(self, backend, res):
+        """ callback when the set of software channels has changed """
+        logging.debug("on_channels_changed %s" % res)
+        if res:
+            self.db.open()
+            # reset the navigation history because software items stored
+            # in the history stack might no longer be available
+            self.available_pane.reset_navigation_history()
+            # refresh the available_pane views to reflect any changes
+            self.available_pane.refresh_apps()
+            self.available_pane.update_app_view()
+            self.update_app_status_menu()
+            self.update_status_bar()
 
     # helper
 
-    # FIXME: move the two functions below into generic code
-    #        and share that with the appdetailsview
-    def _on_trans_finished(self, trans, enum):
-        """callback when a aptdaemon transaction finished"""
-        if enum == aptdaemon.enums.EXIT_FAILED:
-            excep = trans.get_error()
-            msg = "%s: %s\n%s\n\n%s" % (
-                   _("ERROR"),
-                   aptdaemon.enums.get_error_string_from_enum(excep.code),
-                   aptdaemon.enums.get_error_description_from_enum(excep.code),
-                   excep.details)
-            print msg
-        # re-open cache and refresh app display
-        self.cache.open()
     def run_update_cache(self):
         """update the apt cache (e.g. after new sources where added """
-        aptd_client = aptdaemon.client.AptClient()
-        trans = aptd_client.update_cache(exit_handler=self._on_trans_finished)
-        try:
-            trans.run()
-        except dbus.exceptions.DBusException, e:
-            if e._dbus_error_name == "org.freedesktop.PolicyKit.Error.NotAuthorized":
-                pass
-            else:
-                raise
+        self.backend.reload()
 
     def update_app_status_menu(self):
         """Helper that updates the 'File' and 'Edit' menu to enable/disable
@@ -392,10 +433,10 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         """
         logging.debug("update_app_status_menu")
         # check if we have a pkg for this page
-        page = self.notebook_view.get_current_page()
-        try:
-            pkgname = self._selected_pkgname_for_page[page]
-        except KeyError, e:
+        app = None
+        if self.active_pane:
+            app = self.active_pane.get_current_app()
+        if app is None:
             self.menuitem_install.set_sensitive(False)
             self.menuitem_remove.set_sensitive(False)
             self.menuitem_copy_web_link.set_sensitive(False)
@@ -404,20 +445,25 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         if not self.cache.ready:
             glib.timeout_add(100, lambda: self.update_app_status_menu())
             return False
-        # if the pkg is not in the cache, clear menu
-        if not self.cache.has_key(pkgname):
+        # update menu items
+        if (not self.active_pane.is_category_view_showing() and 
+            self.cache.has_key(app.pkgname)):
+            if self.active_pane.app_view.is_action_in_progress_for_selected_app():
+                self.menuitem_install.set_sensitive(False)
+                self.menuitem_remove.set_sensitive(False)
+                self.menuitem_copy_web_link.set_sensitive(False)
+            else:
+                pkg = self.cache[app.pkgname]
+                installed = bool(pkg.installed)
+                self.menuitem_install.set_sensitive(not installed)
+                self.menuitem_remove.set_sensitive(installed)
+                self.menuitem_copy_web_link.set_sensitive(True)
+        else:
+            # clear menu items if category view or if the package is not
+            # in the cache
             self.menuitem_install.set_sensitive(False)
             self.menuitem_remove.set_sensitive(False)
             self.menuitem_copy_web_link.set_sensitive(False)
-        # update File menu status
-        if self.cache.has_key(pkgname):
-            pkg = self.cache[pkgname]
-            installed = bool(pkg.installed)
-            self.menuitem_install.set_sensitive(not installed)
-            self.menuitem_remove.set_sensitive(installed)
-        else:
-            self.menuitem_install.set_sensitive(False)
-            self.menuitem_remove.set_sensitive(False)
         # return False to ensure that a possible glib.timeout_add ends
         return False
 
@@ -430,6 +476,18 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
             # FIXME: deal with the pending view status
             s = ""
         self.label_status.set_text(s)
+        
+    def update_app_list_view(self, channel=None):
+        """Helper that updates the app view list.
+        """
+        if self.active_pane is None:
+            return
+        if channel is None and self.active_pane.is_category_view_showing():
+            return
+        if channel:
+            self.channel_pane.set_channel(channel)
+            self.active_pane.refresh_apps()
+        self.active_pane.update_app_view()
 
     def _on_database_rebuilding_handler(self, is_rebuilding):
         logging.debug("_on_database_rebuilding_handler %s" % is_rebuilding)
@@ -441,7 +499,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         if is_rebuilding:
             self.window_rebuilding.show()
         else:
-            # we need to re-open when the database finished updating
+            # we need to reopen when the database finished updating
             self.db.reopen()
             self.window_rebuilding.hide()
 
@@ -493,8 +551,66 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
             bus_name = dbus.service.BusName('com.ubuntu.Softwarecenter',bus)
             self.dbusControler = SoftwarecenterDbusController(self, bus_name)
 
-    def run(self):
+    def show_available_packages(self, packages):
+        """ Show packages given as arguments in the available_pane
+            If the list of packages is only one element long show that,
+            otherwise turn it into a comma seperated search
+        """
+        if len(packages) == 1:
+            # show a single package
+            pkg_name = packages[0]
+            # FIXME: this currently only works with pkg names for apps
+            #        it needs to perform a search because a App name
+            #        is (in general) not unique
+            app = Application("", pkg_name)
+            self.available_pane.app_details.show_app(app)
+            self.available_pane.notebook.set_current_page(
+                self.available_pane.PAGE_APP_DETAILS)
+        if len(packages) > 1:
+            # turn multiple packages into a search with ","
+            # turn off de-duplication
+            self.available_pane.apps_filter.set_only_packages_without_applications(False)
+            self.available_pane.searchentry.set_text(",".join(packages))
+            self.available_pane.notebook.set_current_page(
+                self.available_pane.PAGE_APPLIST)
+
+    def restore_state(self):
+        if self.config.has_option("general", "size"):
+            (x, y) = self.config.get("general", "size").split(",")
+            self.window_main.set_default_size(int(x), int(y))
+        if (self.config.has_option("general", "maximized") and
+            self.config.getboolean("general", "maximized")):
+            self.window_main.maximize()
+        if (self.config.has_option("general", "available-node-expanded") and
+            self.config.getboolean("general", "available-node-expanded")):
+            self.view_switcher.expand_available_node()
+
+    def save_state(self):
+        logging.debug("save_state")
+        # this happens on a delete event, we explicitely save_state() there
+        if self.window_main.window is None:
+            return
+        if not self.config.has_section("general"):
+            self.config.add_section("general")
+        maximized = self.window_main.window.get_state() & gtk.gdk.WINDOW_STATE_MAXIMIZED
+        if maximized:
+            self.config.set("general", "maximized", "True")
+        else:
+            self.config.set("general", "maximized", "False")
+            # size only matters when non-maximized
+            size = self.window_main.get_size() 
+            self.config.set("general","size", "%s, %s" % (size[0], size[1]))
+        available_node_expanded = self.view_switcher.is_available_node_expanded()
+        if available_node_expanded:
+            self.config.set("general", "available-node-expanded", "True")
+        else:
+            self.config.set("general", "available-node-expanded", "False")
+        self.config.write()
+
+    def run(self, args):
         self.window_main.show_all()
+        self.show_available_packages(args)
+        atexit.register(self.save_state)
         SimpleGtkbuilderApp.run(self)
 
 
